# %%
# Define class to give feature importances
# taken from MIT:
# https://github.com/YousefGh/kmeans-feature-importance


class KMeansInterp(KMeans):
    def __init__(
        self, ordered_feature_names, feature_importance_method="wcss_min", **kwargs
    ):
        super(KMeansInterp, self).__init__(**kwargs)
        self.feature_importance_method = feature_importance_method
        self.ordered_feature_names = ordered_feature_names

    def fit(self, X, y=None, sample_weight=None):
        super().fit(X=X, y=y, sample_weight=sample_weight)

        if not len(self.ordered_feature_names) == self.n_features_in_:
            raise Exception(
                f"Model is fitted on {self.n_features_in_} but ordered_feature_names = {len(self.ordered_feature_names)}"
            )

        if self.feature_importance_method == "wcss_min":
            self.feature_importances_ = self.get_feature_imp_wcss_min()
        elif self.feature_importance_method == "unsup2sup":
            self.feature_importances_ = self.get_feature_imp_unsup2sup(X)
        else:
            raise Exception(
                f" {self.feature_importance_method}"
                + "is not available. Please choose from  ['wcss_min' , 'unsup2sup']"
            )

        return self

    def get_feature_imp_wcss_min(self):
        labels = self.n_clusters
        centroids = self.cluster_centers_
        centroids = np.vectorize(lambda x: np.abs(x))(centroids)
        sorted_centroid_features_idx = centroids.argsort(axis=1)[:, ::-1]

        cluster_feature_weights = {}
        for label, centroid in zip(range(labels), sorted_centroid_features_idx):
            ordered_cluster_feature_weights = centroids[label][
                sorted_centroid_features_idx[label]
            ]
            ordered_cluster_features = [
                self.ordered_feature_names[feature] for feature in centroid
            ]
            cluster_feature_weights[label] = list(
                zip(ordered_cluster_features, ordered_cluster_feature_weights)
            )

        return cluster_feature_weights

    def get_feature_imp_unsup2sup(self, X):
        try:
            from sklearn.ensemble import RandomForestClassifier
        except ImportError as IE:
            print(IE.__class__.__name__ + ": " + IE.message)
            raise Exception(
                "Please install scikit-learn. "
                + "'unsup2sup' method requires using a classifier"
                + "and depends on 'sklearn.ensemble.RandomForestClassifier'"
            )

        cluster_feature_weights = {}
        for label in range(self.n_clusters):
            binary_enc = np.vectorize(lambda x: 1 if x == label else 0)(self.labels_)
            clf = RandomForestClassifier()
            clf.fit(X, binary_enc)

            sorted_feature_weight_idxes = np.argsort(clf.feature_importances_)[::-1]
            ordered_cluster_features = np.take_along_axis(
                np.array(self.ordered_feature_names),
                sorted_feature_weight_idxes,
                axis=0,
            )
            ordered_cluster_feature_weights = np.take_along_axis(
                np.array(clf.feature_importances_), sorted_feature_weight_idxes, axis=0
            )
            cluster_feature_weights[label] = list(
                zip(ordered_cluster_features, ordered_cluster_feature_weights)
            )
        return cluster_feature_weights
