import geopandas as gpd
import numpy as np
import pandas as pd

from justclust.paths import bolzano as paths

# %%
# Define columns and other constant values

quantile_range = (20, 80)

# air quality
aq_raw_cols = ["Area_r1", "Area_r2", "Area_r3", "Area_r4", "Area_r6", "Area_r9"]
aq_nor_cols = [
    "P_Area_r1",
    "P_Area_r2",
    "P_Area_r3",
    "P_Area_r4",
    "P_Area_r6",
    "P_Area_r9",
]

# carbon
# absorption
co2_raw_cols = ["CO2"]
co2_nor_cols = ["kgCO2/m²"]

# emmissions
ems = ["gasolio_em", "gpl_em", "ee_em", "metano_em", "legno_em", "tot_em", "c_ee_em"]
co2_em_raw_cols = ["finale"]
co2_em_nor_cols = ["finale/m²"]

# flora fauna and habitat
ffh_raw_cols = ["F_Area_m2"]
ffh_nor_cols = ["P_Prot_are"]

# spatial
spt_ag_raw_cols = ["Area_300m_", "Area_750m_"]
spt_ag_nor_cols = ["P_area_300", "P_area_750"]

spt_wk_nor_cols = ["MEAN_Resul"]

# temporary
tem_raw_cols = ["Area_diff"]
tem_nor_cols = ["P_Area_dif"]

# thermal
thm_nor_cols = ["mean_norm"]

# socio-demographic
socio_abs_cols = [
    "Maschi",
    "Femmine",
    "Totale popolazione",
    "0-14",
    "15-64",
    "65 e oltre",
    "indice di vecchiaia",
    "0-17",
    "Stranieri",
    "Media componenti",
    "Totale famiglie con figli",
    "Totale famiglie senza figli",
    "Totale famiglie",
    "Single",
]
socio_perc_cols = [
    "%65+",
    "%minori",
    "%stranieri",
    "%con figli",
    "%senza figli",
    "%Single",
]

sez_cols = ["geometry", "SEZ", "SEZ2011", "COD_REG", "COD_ISTAT", "PRO_COM"]

# %%
## SELECT COLUMNS TO BE USED

cols = [
    # air-quality: % risk based on SVF and road classification
    "P_Area_r1",
    "P_Area_r2",
    "P_Area_r3",
    "P_Area_r4",
    "P_Area_r6",
    "P_Area_r9",
    # carbon: absorption and emmissions
    "kgCO2/m²",
    "finale/m²",
    # FFH: % protected area, % to 5 min to prot. areas
    "P_Prot_are",
    # spatial: % of area within a distance from protected areas and walkability index
    "P_area_300",
    # "P_area_750",
    "MEAN_Resul",
    # temporal: % area changed
    "P_Area_dif",
    # thermal
    "mean_norm",
    # socio demographic
    # "%65+",
    # "%minori",
    "indice di vecchiaia",
    "%stranieri",
    "%con figli",
    "%senza figli",
    "%Single",
]

# define weights
wcols = np.array(
    [
        # air-quality: % risk based on SVF and road classification
        1.0 / 6.0,
        1.0 / 6.0,
        1.0 / 6.0,
        1.0 / 6.0,
        1.0 / 6.0,
        1.0 / 6.0,
        # carbon: absorption and emmissions
        1.0 / 2.0,
        1.0 / 2.0,
        # FFH: % protected area, % to 5 min to prot. areas
        1.0,
        # spatial: % of area within a distance from protected areas and walkability index
        1.0 / 2.0,
        # "P_area_750",
        1.0 / 2.0,
        # temporal: % area changed
        1.0,
        # thermal
        1.0,
        # socio demographic
        # "%65+",
        # "%minori",
        1.0 / 5.0,
        1.0 / 5.0,
        1.0 / 5.0,
        1.0 / 5.0,
        1.0 / 5.0,
    ],
    dtype=float,
)


# %%
# Define function to read files


def read_data():
    aq_scr = gpd.read_file(paths.aq_scr_shp)
    co2_ab = gpd.read_file(paths.co2_ab_shp)
    co2_ab["kgCO2/m²"] = co2_ab["CO2"] / co2_ab.area
    co2_em = gpd.read_file(paths.co2_em_shp)
    co2_em["finale/m²"] = co2_em["finale"] / co2_em.area
    ffh_pa = gpd.read_file(paths.ffh_pa_shp)
    spt_ag = gpd.read_file(paths.spt_ag_shp)
    spt_wk = gpd.read_file(paths.spt_wk_shp)
    tem_ss = gpd.read_file(paths.tem_ss_shp)
    thm_sh = gpd.read_file(paths.thm_sh_shp)

    socio_pk = gpd.read_file(paths.socio_gpkg, driver="GPKG")
    # fix column
    res = []
    for val in socio_pk["indice di vecchiaia"]:
        try:
            v = float(val.strip().replace(",", "."))
        except Exception:
            v = np.nan
        res.append(v)
    socio_pk["indice di vecchiaia"] = np.array(res, dtype=float)

    raw = pd.concat(
        [
            co2_ab.loc[:, sez_cols],
            aq_scr.loc[:, aq_raw_cols + aq_nor_cols],
            co2_ab.loc[:, co2_raw_cols + co2_nor_cols],
            co2_em.loc[:, co2_em_raw_cols + co2_em_nor_cols + ems],
            ffh_pa.loc[:, ffh_raw_cols + ffh_nor_cols],
            spt_ag.loc[:, spt_ag_raw_cols + spt_ag_nor_cols],
            spt_wk.loc[:, spt_wk_nor_cols],
            tem_ss.loc[:, tem_raw_cols + tem_nor_cols],
            thm_sh.loc[:, thm_nor_cols],
            socio_pk.loc[:, socio_abs_cols + socio_perc_cols],
        ],
        axis="columns",
    )
    return raw
